import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hub/shared/components/molecules/locals/index.dart';
import 'package:hub/shared/components/molecules/modules/index.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';
import 'package:hub/shared/helpers/storage/storage_helper.dart';
import 'package:hub/shared/utils/dialog_util.dart';
import 'package:rxdart/rxdart.dart';

class LocalsRepositoryImpl implements LocalsRepository {
  final LocalsRemoteDataSource remoteDataSource = LocalsRemoteDataSourceImpl();

  static final license = BehaviorSubject<LicenseStatus>();
  static get stream => license.stream;

  @override
  Future<void> validateLocal(BuildContext context) async {
    return await remoteDataSource.checkLocals(context);
  }

  @override
  Future<bool> update(BuildContext context) async {
    bool response = false;
    while (!response) {
      response = await remoteDataSource.processLocals(context);
      if (!response) {
        await StorageHelper().set(KeychainStorageKey.clientUUID.value, '');
        const String errorMsg = 'Erro ao atualizar locais';
        await DialogUtil.error(context, errorMsg);
      }
    }
    context.read<LocalProfileBloc>().add(LocalProfileEvent());
    return response;
  }

  @override
  Future<void> unlinkLocal(BuildContext context) {
    return remoteDataSource.detachLocal(context);
  }
}
